#!/bin/bash
#
# Copyright (C) 2014-2018 NetFlow Logic. All rights reserved.
#
# External Data Feeder for NFO Daemon install/uninstall script
#
#==============================================================
# Options:
# -i - install External Data Feeder for NFO daemon
# -u - uninstall External Data Feeder for NFO daemon
#==============================================================

declare -i e_INCORRECT_ARG=70
declare -i e_UNSUPPORTED_ID=71
declare -i e_ALREADY_INSTALLED=72
declare -i e_NOT_INSTALLED=73
declare -i e_CANNOT_DETECT_DISTRIBUTOR_ID=74
declare -i e_SYSTEMD_DIR_FOR_SERVICES_NOT_FOUND=75

ARG0="${0}"
while [ -h "${ARG0}" ]; do
	ls=$(ls -ld "${ARG0}")
	link=$(expr "${ls}" : '.*-> \(.*\)$')
	if $(expr "${link}" : '/.*' > /dev/null); then
		ARG0="${link}"
	else
		ARG0="$(dirname ${ARG0})/${link}"
	fi
done

function isSystemd
{
	local -i __errCode=1

	whereis systemctl | grep "^systemctl:[[:space:]]\+.*systemctl.*$" > /dev/null
	__errCode=$?

	return ${__errCode}
}

function isOpenRC
{
	local -i __errCode=1

	whereis rc-service | grep "^rc\-service:[[:space:]]\+.*rc\-service.*$" > /dev/null
	__errCode=$?

	return ${__errCode}
}

declare -i openRCFound=1
declare -i systemdFound=1

declare appRoot="$(readlink -f $(dirname ${ARG0}))"

# Mode: install or uninstall
declare mode=""
declare msgAlreadyInstalled="External Data Feeder for NFO daemon is already installed."
declare msgNotInstalled="External Data Feeder for NFO daemon was not installed."
declare msgCannotDetectDistributorId="Cannot detect Distributor ID."
declare msgSystemdForServicesDirNotFound="Systemd directory for services not found."
declare msgUseInitSystem=""
declare msgOperation=""
declare msgOperationSuccess=""
declare msgOperationFail=""
declare msgUnsupportedDistributorId=""

isSystemd
systemdFound=$?

isOpenRC
openRCFound=$?

for arg in "${*}"
do
	case ${arg} in
		-i)
			mode="install"
			msgOperation="Installing External Data Feeder for NFO daemon..."
			msgOperationSuccess="External Data Feeder for NFO daemon has been successfully installed."
			msgOperationFail="External Data Feeder for NFO daemon install FAILED! Error code -- "
			msgUnsupportedDistributorId="External Data Feeder for NFO daemon install is not supported for Distributor ID '"
			if (( ${systemdFound} == 0 ));then
				msgUseInitSystem="Systemd found. Using it for registering External Data Feeder for NFO daemon."
			elif (( ${openRCFound} == 0 ));then
				msgUseInitSystem="OpenRC found. Using it for registering External Data Feeder for NFO daemon."
			else
				msgUseInitSystem="Systemd not found. Using UNIX System V for registering External Data Feeder for NFO daemon."
			fi
		;;
		-u)
			mode="uninstall"
			msgOperation="Uninstalling External Data Feeder for NFO daemon..."
			msgOperationSuccess="External Data Feeder for NFO daemon has been successfully uninstalled."
			msgOperationFail="External Data Feeder for NFO daemon uninstall FAILED! Error code -- "
			msgUnsupportedDistributorId="External Data Feeder for NFO daemon uninstall is not supported for Distributor ID '"
			if (( ${systemdFound} == 0 ));then
				msgUseInitSystem="Systemd found. Using it for unregistering External Data Feeder for NFO daemon."
			elif (( ${openRCFound} == 0 ));then
				msgUseInitSystem="OpenRC found. Using it for unregistering External Data Feeder for NFO daemon."
			else
				msgUseInitSystem="Systemd not found. Using UNIX System V for unregistering External Data Feeder for NFO daemon."
			fi
		;;
		*)
			echo "Script usage:"
			echo "${0} -i | -u"
			echo "-i - install External Data Feeder for NFO daemon"
			echo "-u - uninstall External Data Feeder for NFO daemon"
			exit ${e_INCORRECT_ARG}
		;;
	esac
done


declare jreHome="${appRoot}/java/jre8"
declare jreHomeEnv=""
declare jreTARName=""

function checkLastOperationResult 
{
	local __res=$?
	if [[ ${__res} != 0 ]] ; then
		echo -e '\E[31;40m'"${msgOperationFail}${__res}."
		tput sgr0
		exit ${__res}
	fi
}

# main part

pushd ${appRoot} >> /dev/null

echo "${msgUseInitSystem}"
echo "${msgOperation}"

if (( ${systemdFound} == 0 ));then
	case ${mode} in
		install)
			if [[ ! -d "/etc/systemd/system" ]]; then
				echo "${msgSystemdForServicesDirNotFound}"
				exit ${e_SYSTEMD_DIR_FOR_SERVICES_NOT_FOUND}
			elif [[ ! -f "/etc/systemd/system/nfi_updd.service" ]]; then
				declare vTmp="${appRoot//\//\\/}"
				vTmp="${vTmp//\./\\.}"
				cat ${appRoot}/nfi_updd.service.template | sed "s/\[nfou_app_root\]/"${vTmp}"/" > /etc/systemd/system/nfi_updd.service
				checkLastOperationResult
				systemctl daemon-reload 2>&1 > /dev/null
			else
				echo "${msgAlreadyInstalled}"
				exit ${e_ALREADY_INSTALLED}
			fi
		;;
		uninstall)
			if [[ -f "/etc/systemd/system/nfi_updd.service" ]]; then
				systemctl disable nfi_updd.service > /dev/null
				checkLastOperationResult
				rm /etc/systemd/system/nfi_updd.service 2>&1 > /dev/null
				checkLastOperationResult
				systemctl daemon-reload 2>&1 > /dev/null
				systemctl reset-failed 2>&1 > /dev/null
			else
				echo "${msgNotInstalled}"
				exit ${e_NOT_INSTALLED}
			fi
		;;
	esac
else
	# Distributor ID:
	# rhel -- RHEL, CentOS or Oracle Linux
	# debian -- Debian or Ubuntu
	# suse -- SUSE Linux Enterprise Server
	# gentoo -- Gentoo Linux
	declare id=""
	is_lsb_release=$(whereis lsb_release | grep -c "lsb_release:.\+")
	if (( is_lsb_release == 0 ));then
		is_os_release=$(find /etc/ -maxdepth 1 -mindepth 1 -type f -name 'os-release' | grep -c "\/etc\/os-release")
		if (( is_os_release == 0 ));then
			is_etc_release=$(find /etc/ -maxdepth 1 -mindepth 1 -type f -name '*-release' | grep -c "\/etc\/.*-release")
			if (( is_etc_release == 0 ));then
				if [[ -f "/etc/debian_version" ]]; then
					strId="debian"
				else
					echo "${msgCannotDetectDistributorId}"
					exit ${e_CANNOT_DETECT_DISTRIBUTOR_ID}
				fi
			else
				sId=$(sed -n '1p' /etc/*-release)
				strId=$(echo "${sId}" | tr [[:upper:]] [[:lower:]] | sed "s/[[:space:]]//g")
			fi
		else
			sId=$(sed -n '/^ID\=/p;' /etc/os-release | sed 's/ID\=\"\?\([[:alnum:]]\+\)\"\?$/\1/')
			strId="${sId}"
		fi
	else
		sId=$(lsb_release -i | sed "s/Distributor[[:space:]]*ID:[[:space:]]*//")
		strId=$(echo "${sId}" | tr [[:upper:]] [[:lower:]])
	fi

	case ${strId} in
		redhat*) id="rhel";;
		rhel*) id="rhel";;
		centos*) id="rhel";;
		ol) id="rhel";;
		oracle*) id="rhel";;
		debian*) id="debian";;
		ubuntu*) id="debian";;
		suse*) id="suse";;
		opensuse*) id="suse";;
		sles*) id="suse";;
		gentoo*) id="gentoo";;
		*)
			echo "${msgUnsupportedDistributorId}${sId}"
			exit ${e_UNSUPPORTED_ID}
		;;
	esac

	declare -i cnt=0

	case ${id} in
		rhel)
			cnt=$(/sbin/chkconfig --list nfi_updd 2>&1 | grep -c "^nfi_updd[[:space:]]\+0:\(off\|on\)[[:space:]]\+1:\(off\|on\)[[:space:]]\+2:\(off\|on\)[[:space:]]\+3:\(off\|on\)[[:space:]]\+4:\(off\|on\)[[:space:]]\+5:\(off\|on\)[[:space:]]\+6:\(off\|on\)")

			case ${mode} in
				install)
					if [[ ! -h "/etc/init.d/nfi_updd" ]]; then
						ln -s ${appRoot}/bin/nfiu_daemon_rhel.sh /etc/init.d/nfi_updd 2>&1 > /dev/null
						checkLastOperationResult
					else
						if [[ ${cnt} -eq 1 ]]; then
							echo "${msgAlreadyInstalled}"
							exit ${e_ALREADY_INSTALLED}
						fi
					fi

					if [[ ${cnt} -eq 1 ]]; then
						/sbin/chkconfig --del nfi_updd 2>&1 > /dev/null
						checkLastOperationResult
					fi
					/sbin/chkconfig --add nfi_updd 2>&1 > /dev/null
					checkLastOperationResult
				;;
				uninstall)
					if [[ -h "/etc/init.d/nfi_updd" ]]; then
						if [[ ${cnt} -eq 1 ]]; then
							/sbin/chkconfig --del nfi_updd 2>&1 > /dev/null
							checkLastOperationResult
						fi
						rm /etc/init.d/nfi_updd
					else
						echo "${msgNotInstalled}"
						exit ${e_NOT_INSTALLED}
					fi
				;;
			esac
		;;
		debian)
			cnt2=$(ls /etc/rc2.d/S* | grep -c "S*nfi_updd$")
			cnt3=$(ls /etc/rc3.d/S* | grep -c "S*nfi_updd$")
			cnt4=$(ls /etc/rc4.d/S* | grep -c "S*nfi_updd$")
			cnt5=$(ls /etc/rc5.d/S* | grep -c "S*nfi_updd$")
			((cnt = cnt2 + cnt3 + cnt4 + cnt5))

			case ${mode} in
				install)
					if [[ ! -h "/etc/init.d/nfi_updd" ]]; then
						ln -s ${appRoot}/bin/nfiu_daemon_deb.sh /etc/init.d/nfi_updd 2>&1 > /dev/null
						checkLastOperationResult
					else
						if [[ ${cnt} -ne 0 ]]; then
							echo "${msgAlreadyInstalled}"
							exit ${e_ALREADY_INSTALLED}
						fi
					fi

					if [[ ${cnt} -gt 0 ]]; then
						update-rc.d -f nfi_updd remove 2>&1 > /dev/null
						checkLastOperationResult
					fi
					update-rc.d nfi_updd start 85 2 3 4 5 . stop 01 0 1 6 . 2>&1 > /dev/null
					checkLastOperationResult
				;;
				uninstall)
					if [[ -h "/etc/init.d/nfi_updd" ]]; then
						if [[ ${cnt} -gt 0 ]]; then
							update-rc.d -f nfi_updd remove 2>&1 > /dev/null
							checkLastOperationResult
						fi
						rm /etc/init.d/nfi_updd
					else
						echo "${msgNotInstalled}"
						exit ${e_NOT_INSTALLED}
					fi
				;;
			esac
		;;
		suse)
			cnt=$(/sbin/chkconfig --list nfi_updd 2>&1 | grep -c "^nfi_updd[[:space:]]\+0:\(off\|on\)[[:space:]]\+1:\(off\|on\)[[:space:]]\+2:\(off\|on\)[[:space:]]\+3:\(off\|on\)[[:space:]]\+4:\(off\|on\)[[:space:]]\+5:\(off\|on\)[[:space:]]\+6:\(off\|on\)")

			case ${mode} in
				install)
					if [[ ! -h "/etc/init.d/nfi_updd" ]]; then
						ln -s ${appRoot}/bin/nfiu_daemon_suse.sh /etc/init.d/nfi_updd 2>&1 > /dev/null
						checkLastOperationResult
					else
						if [[ ${cnt} -eq 1 ]]; then
							echo "${msgAlreadyInstalled}"
							exit ${e_ALREADY_INSTALLED}
						fi
					fi

					if [[ ${cnt} -eq 1 ]]; then
						/sbin/chkconfig --del nfi_updd 2>&1 > /dev/null
						checkLastOperationResult
					fi
					/sbin/chkconfig --add nfi_updd 2>&1 > /dev/null
					checkLastOperationResult
				;;
				uninstall)
					if [[ -h "/etc/init.d/nfi_updd" ]]; then
						if [[ ${cnt} -eq 1 ]]; then
							/sbin/chkconfig --del nfi_updd 2>&1 > /dev/null
							checkLastOperationResult
						fi
						rm /etc/init.d/nfi_updd 2>&1 > /dev/null
					else
						echo "${msgNotInstalled}"
						exit ${e_NOT_INSTALLED}
					fi
				;;
			esac
		;;
		gentoo)
			cnt=$(rc-service -r nfi_updd 2>&1 | grep -c "^\/etc\/init\.d\/nfi_updd$")

			case ${mode} in
				install)
					if [[ ! -h "/etc/init.d/nfi_updd" ]]; then
						ln -s ${appRoot}/bin/nfiu_daemon_gentoo.sh /etc/init.d/nfi_updd 2>&1 > /dev/null
						checkLastOperationResult
					else
						if [[ ${cnt} -eq 1 ]]; then
							echo "${msgAlreadyInstalled}"
							exit ${e_ALREADY_INSTALLED}
						fi
					fi

					if [[ ${cnt} -eq 1 ]]; then
						rc-update del nfi_updd 2>&1 > /dev/null
						checkLastOperationResult
					fi
					rc-update add nfi_updd default 2>&1 > /dev/null
					checkLastOperationResult
				;;
				uninstall)
					if [[ -h "/etc/init.d/nfi_updd" ]]; then
						if [[ ${cnt} -eq 1 ]]; then
							rc-update del nfi_updd 2>&1 > /dev/null
							checkLastOperationResult
						fi
						rm /etc/init.d/nfi_updd
					else
						echo "${msgNotInstalled}"
						exit ${e_NOT_INSTALLED}
					fi
				;;
			esac
		;;
	esac
fi

popd > /dev/null

echo "${msgOperationSuccess}"
