#!/bin/bash
#
# Copyright (C) 2014-2019 NetFlow Logic. All rights reserved.
#
# External Data Feeder for NFO setup script
#
#==============================================================
# Options:
# -i - install External Data Feeder for NFO
# -u - uninstall External Data Feeder for NFO
#==============================================================

declare -i e_INCORRECT_ARG=70
declare -i e_ALREADY_INSTALLED=72
declare -i e_NOT_INSTALLED=73
declare -i e_NFI_STARTED=75

ARG0="${0}"
while [ -h "${ARG0}" ]; do
	ls=$(ls -ld "${ARG0}")
	link=$(expr "${ls}" : '.*-> \(.*\)$')
	if $(expr "${link}" : '/.*' > /dev/null); then
		ARG0="${link}"
	else
		ARG0="$(dirname ${ARG0})/${link}"
	fi
done

declare -i openRCFound=1
declare -i systemdFound=1

declare appRoot="$(readlink -f $(dirname ${ARG0}))"

declare mode=""
declare jreTARName="zulu8.38.0.13-ca-jre8.0.212-linux_x64.tar.gz"

declare msgConfiguring=""
declare msgConfiguringSuccess=""
declare msgConfiguringFail=""

declare msgNFIUStart="Starting External Data Feeder for NFO..."
declare msgDaemonRunAdvice="Your Linux OS doesn't support systemd. \
To install/uninstall External Data Feeder for NFO service on RHEL, CentOS, Oracle Linux, \
SUSE, Debian or Ubuntu, please use commands '${appRoot}/register_daemon.sh -i' or \
'${appRoot}/register_daemon.sh -u' respectively."
declare msgSystemdDaemonRunAdvice="Your Linux OS supports systemd. \
To install/uninstall External Data Feeder for NFO service, please use commands '${appRoot}/register_daemon.sh -i' or \
'${appRoot}/register_daemon.sh -u' respectively."
declare msgOpenRCDaemonRunAdvice="Your Linux OS supports OpenRC. \
To install/uninstall External Data Feeder for NFO service, please use commands '${appRoot}/register_daemon.sh -i' or \
'${appRoot}/register_daemon.sh -u' respectively."
declare msgNFIURunAdvice="To get all available commands for External Data Feeder for NFO service, please \
use command '/etc/init.d/nfi_updd'."
declare msgSystemdNFIURunAdvice="To manage External Data Feeder for NFO service, please \
use command 'systemctl start|stop|restart|status nfi_updd.service'. \
For details, type 'systemctl --help'."
declare msgOpenRCNFIURunAdvice="To manage External Data Feeder for NFO service, please \
use command 'rc-service nfi_updd start|stop|restart|status'. \
For details, type 'rc-service --help'."
declare msgAlreadyInstalled="External Data Feeder for NFO is already installed."

function checkLastOperationResult
{
	local __res=$?
	if [[ ${__res} != 0 ]] ; then
		echo -e '\E[31;40m'"${msgConfiguringFail}${__res}."
		tput sgr0
		exit ${__res}
	fi
}

function isSystemd
{
	local -i __errCode=1

	whereis systemctl | grep "^systemctl:[[:space:]]\+.*systemctl.*$" > /dev/null
	__errCode=$?

	return ${__errCode}
}

function isOpenRC
{
	local -i __errCode=1

	whereis rc-service | grep "^rc\-service:[[:space:]]\+.*rc\-service.*$" > /dev/null
	__errCode=$?

	return ${__errCode}
}

# main part
for arg in "${*}"
do
	case ${arg} in
		-i)
			mode="install"
			msgConfiguring="Installing External Data Feeder for NFO..."
			msgConfiguringSuccess="External Data Feeder for NFO has been successfully installed."
			msgConfiguringFail="External Data Feeder for NFO install FAILED! Error code -- "
		;;
		-u)
			mode="uninstall"
			msgConfiguring="Uninstalling External Data Feeder for NFO..."
			msgConfiguringSuccess="External Data Feeder for NFO has been successfully uninstalled."
			msgConfiguringFail="External Data Feeder for NFO uninstall FAILED! Error code -- "
		;;
		*)
			echo "Script usage:"
			echo "${0} -i | -u"
			echo "-i - install External Data Feeder for NFO"
			echo "-u - uninstall External Data Feeder for NFO"
			exit ${e_INCORRECT_ARG}
		;;
	esac
done

pushd ${appRoot} >> /dev/null

echo "${msgConfiguring}"

isSystemd
systemdFound=$?

if (( ${systemdFound} == 0 ));then
	if [[ -f "/etc/systemd/system/nfi_updd.service" ]]; then
		systemctl is-active nfi_updd.service | grep -c '^active$' 2>&1 > /dev/null
		is_jsvc_running=$?
		if (( ${is_jsvc_running} == 0 )); then
			systemctl stop nfi_updd.service
		fi
	fi
else
	isOpenRC
	openRCFound=$?

	if [[ -h "/etc/init.d/nfi_updd" ]]; then
		/etc/init.d/nfi_updd status | grep -c '^External Data Feeder for NFO daemon is running.*$' 2>&1 > /dev/null
		is_jsvc_running=$?
		if (( ${is_jsvc_running} == 0 )); then
			/etc/init.d/nfi_updd stop
			checkLastOperationResult
		fi
	fi
fi

case ${mode} in
	install)

		if [[ -d "java/jre7/" ]];then
			rm -rf "java/jre7/" >> /dev/null
		fi
		if [[ -d "java/jre8/" ]];then
			rm -rf "java/jre8/" >> /dev/null
		fi

		tar zxvf java/${jreTARName} -C java --no-same-owner >> /dev/null
		checkLastOperationResult
		jdkDir=$(tar tzf java/${jreTARName} | head -1 | sed -e 's/\/.*//')
		checkLastOperationResult
		mv java/${jdkDir}/ java/jre8/ >> /dev/null
		checkLastOperationResult

		./register_daemon.sh -i
		retVal=$?
		if [[ ${retVal} -eq 0 ]]; then
			:
		elif [[ ${retVal} -eq ${e_ALREADY_INSTALLED} ]]; then
			./register_daemon.sh -u
			checkLastOperationResult
			./register_daemon.sh -i
			checkLastOperationResult
		else
			echo -e '\E[31;40m'"${msgConfiguringFail}${retVal}."
			tput sgr0
			exit ${retVal}
		fi

		echo "${msgConfiguringSuccess}"

		echo "${msgNFIUStart}"
		if (( ${systemdFound} == 0 ));then
			systemctl start nfi_updd.service
			checkLastOperationResult
			systemctl enable nfi_updd.service 2>&1 > /dev/null
		else
			/etc/init.d/nfi_updd start
		fi
		checkLastOperationResult

		if (( ${systemdFound} == 0 ));then
			echo -e '\E[32;40m'"${msgSystemdDaemonRunAdvice}"
			echo "${msgSystemdNFIURunAdvice}"
		elif (( ${openRCFound} == 0 ));then
			echo -e '\E[32;40m'"${msgOpenRCDaemonRunAdvice}"
			echo "${msgOpenRCNFIURunAdvice}"
		else
			echo -e '\E[32;40m'"${msgDaemonRunAdvice}"
			echo "${msgNFIURunAdvice}"
		fi
		tput sgr0
	;;
	uninstall)
		./register_daemon.sh -u
		retVal=$?
		if [[ ${retVal} -eq 0 || ${retVal} -eq ${e_NOT_INSTALLED} ]]; then
			# Use bash instead sh (see code backup-conf.sh) to avoid the issues in DASH (Ubuntu, Debian)
			bash ./backup-conf.sh
		else
			echo -e '\E[31;40m'"${msgConfiguringFail}${retVal}."
			tput sgr0
			exit ${retVal}
		fi

		target=$(basename ${appRoot})
		if [[ "${target}" == "nfi-updater" ]]; then
			for d in $(find ${appRoot}/ -maxdepth 1 -type d -path "${appRoot}/*")
			do
				base=$(basename ${d})
				case ${base} in
					nfi-updater)
						find ${d}/ -maxdepth 1 -type f -name "*" -exec rm -f {} \;
					;;
					logs | conf | conf-backup)
						:
					;;
					*)
						rm -rf "${d}"
					;;
				esac
			done
		fi
		echo "${msgConfiguringSuccess}"
	;;
esac

popd > /dev/null
